@@ -3,9 +3,6 @@
 # Copyright (c) CERN, 2021.                 #
 # ######################################### #
 
-from pathlib import Path
-from tkinter import N
-
 import numpy as np
 from scipy.special import factorial
 
@@ -104,13 +101,15 @@ class Elens(BeamElement):
 # if array is needed we do it like this
 #    _xofields={'inner_radius': xo.Float64[:]}
     _xofields={
-               'current':      xo.Float64,
-               'inner_radius': xo.Float64,
-               'outer_radius': xo.Float64,
-               'elens_length': xo.Float64,
-               'voltage':      xo.Float64,
-               'residual_kick_x': xo.Float64,
-               'residual_kick_y': xo.Float64
+               'current'                : xo.Float64,
+               'inner_radius'           : xo.Float64,
+               'outer_radius'           : xo.Float64,
+               'elens_length'           : xo.Float64,
+               'voltage'                : xo.Float64,
+               'residual_kick_x'        : xo.Float64,
+               'residual_kick_y'        : xo.Float64,
+               'coefficients_polynomial': xo.Float64[:],
+               'polynomial_order'       : xo.Float64
               }
 
     _extra_c_sources = [
@@ -123,8 +122,12 @@ class Elens(BeamElement):
                         voltage      = 0.,
                         residual_kick_x = 0,
                         residual_kick_y = 0,
+                        coefficients_polynomial = [0],
                         _xobject = None,
                         **kwargs):
+
+        kwargs["coefficients_polynomial"] = len(coefficients_polynomial)
+
         if _xobject is not None:
             super().__init__(_xobject=_xobject)
         else:
@@ -137,6 +140,11 @@ class Elens(BeamElement):
             self.residual_kick_x   = residual_kick_x
             self.residual_kick_y   = residual_kick_y
 
+            self.coefficients_polynomial[:] = self._arr2ctx(coefficients_polynomial)
+            polynomial_order = len(coefficients_polynomial)-1
+            self.polynomial_order = polynomial_order
+
+
     def get_backtrack_element(self, _context=None, _buffer=None, _offset=None):
         return self.__class__(
                               current=self.current,
@@ -144,6 +152,8 @@ class Elens(BeamElement):
                               outer_radius=self.outer_radius,
                               elens_length=-self.elens_length,
                               voltage=self.voltage,
+                              coefficients_polynomial = self.coefficients_polynomial,
+                              polynomial_order = self.polynomial_order,
                               _context=_context, _buffer=_buffer, _offset=_offset)
 
 
@@ -155,6 +165,9 @@ class Wire(BeamElement):
                'current': xo.Float64,
                'xma'    : xo.Float64,
                'yma'    : xo.Float64,
+
+               'post_subtract_px': xo.Float64,
+               'post_subtract_py': xo.Float64,
               }
 
     _extra_c_sources = [
@@ -167,8 +180,11 @@ class Wire(BeamElement):
                         current = 0,
                         xma     = 0,
                         yma     = 0,
+                        post_subtract_px = 0,
+                        post_subtract_py = 0,
                         _xobject = None,
                         **kwargs):
+
         if _xobject is not None:
             super().__init__(_xobject=_xobject)
         else:
@@ -178,15 +194,12 @@ class Wire(BeamElement):
             self.current = current
             self.xma     = xma
             self.yma     = yma
+            self.post_subtract_px = post_subtract_px
+            self.post_subtract_py = post_subtract_py
+
 
     def get_backtrack_element(self, _context=None, _buffer=None, _offset=None):
-        return self.__class__(
-                              L_phy   = self.L_phy,
-                              L_int   = self.L_int,
-                              current = -self.current,
-                              xma     = self.xma,
-                              yma     = self.yma,
-                              _context=_context, _buffer=_buffer, _offset=_offset)
+        raise NotImplementedError
 
 
 class SRotation(BeamElement):
@@ -552,8 +565,11 @@ class LinearTransferMatrix(BeamElement):
         'damping_factor_s':xo.Float64,
         'uncorrelated_gauss_noise': xo.Int64,
         'gauss_noise_ampl_x':xo.Float64,
+        'gauss_noise_ampl_px':xo.Float64,
         'gauss_noise_ampl_y':xo.Float64,
-        'gauss_noise_ampl_s':xo.Float64,
+        'gauss_noise_ampl_py':xo.Float64,
+        'gauss_noise_ampl_zeta':xo.Float64,
+        'gauss_noise_ampl_delta':xo.Float64,
         }
 
     _extra_c_sources = [
@@ -574,7 +590,7 @@ class LinearTransferMatrix(BeamElement):
                      y_ref_0 = 0.0, py_ref_0 = 0.0, y_ref_1 = 0.0, py_ref_1 = 0.0,
                      damping_rate_x = 0.0, damping_rate_y = 0.0, damping_rate_s = 0.0,
                      equ_emit_x = 0.0, equ_emit_y = 0.0, equ_emit_s = 0.0,
-                     gauss_noise_ampl_x=0.0,gauss_noise_ampl_y=0.0,gauss_noise_ampl_s=0.0,
+                     gauss_noise_ampl_x=0.0,gauss_noise_ampl_px=0.0,gauss_noise_ampl_y=0.0,gauss_noise_ampl_py=0.0,gauss_noise_ampl_zeta=0.0,gauss_noise_ampl_delta=0.0,
                      **nargs):
 
         if (chroma_x==0 and chroma_y==0
@@ -654,21 +670,46 @@ class LinearTransferMatrix(BeamElement):
             raise ValueError('Equilibrium emittances cannot be negative')
         nargs['uncorrelated_gauss_noise'] = False
         nargs['gauss_noise_ampl_x'] = 0.0
+        nargs['gauss_noise_ampl_px'] = 0.0
         nargs['gauss_noise_ampl_y'] = 0.0
-        nargs['gauss_noise_ampl_s'] = 0.0
-        if equ_emit_x > 0.0 or equ_emit_y > 0.0 or equ_emit_s > 0.0:
-            nargs['uncorrelated_gauss_noise'] = True
-            nargs['gauss_noise_ampl_x'] = np.sqrt(2.0*equ_emit_x*damping_rate_x/beta_x_1)
-            nargs['gauss_noise_ampl_y'] = np.sqrt(2.0*equ_emit_y*damping_rate_y/beta_y_1)
-            nargs['gauss_noise_ampl_s'] = np.sqrt(2.0*equ_emit_s*damping_rate_s/beta_s)
+        nargs['gauss_noise_ampl_py'] = 0.0
+        nargs['gauss_noise_ampl_zeta'] = 0.0
+        nargs['gauss_noise_ampl_delta'] = 0.0
 
-        if gauss_noise_ampl_x < 0.0 or gauss_noise_ampl_y < 0.0 or gauss_noise_ampl_s < 0.0:
-            raise ValueError('Gaussian noise amplitude cannot be negative')
-        if gauss_noise_ampl_x > 0.0 or gauss_noise_ampl_y > 0.0 or gauss_noise_ampl_s > 0.0:
+        assert equ_emit_x >= 0.0
+        assert equ_emit_y >= 0.0
+        assert equ_emit_s >= 0.0
+
+        if equ_emit_x > 0.0:
+            assert alpha_x_1 == 0
+            nargs['uncorrelated_gauss_noise'] = True
+            nargs['gauss_noise_ampl_px'] = np.sqrt(equ_emit_x*damping_rate_x/beta_x_1)
+            nargs['gauss_noise_ampl_x'] = beta_x_1*nargs['gauss_noise_ampl_px']
+        if equ_emit_y > 0.0:
+            assert alpha_y_1 == 0
+            nargs['uncorrelated_gauss_noise'] = True
+            nargs['gauss_noise_ampl_py'] = np.sqrt(equ_emit_y*damping_rate_y/beta_y_1)
+            nargs['gauss_noise_ampl_y'] = beta_y_1*nargs['gauss_noise_ampl_py']
+        if equ_emit_s > 0.0:
+            nargs['uncorrelated_gauss_noise'] = True
+            nargs['gauss_noise_ampl_delta'] = np.sqrt(equ_emit_s*damping_rate_s/beta_s)
+            nargs['gauss_noise_ampl_zeta'] = beta_s*nargs['gauss_noise_ampl_delta']
+
+        assert gauss_noise_ampl_x >= 0.0
+        assert gauss_noise_ampl_px >= 0.0
+        assert gauss_noise_ampl_y >= 0.0
+        assert gauss_noise_ampl_py >= 0.0
+        assert gauss_noise_ampl_zeta >= 0.0
+        assert gauss_noise_ampl_delta >= 0.0
+
+        if gauss_noise_ampl_x > 0.0 or gauss_noise_ampl_px > 0.0 or gauss_noise_ampl_y > 0.0 or gauss_noise_ampl_py > 0.0 or gauss_noise_ampl_zeta > 0.0 or gauss_noise_ampl_delta > 0.0:
             nargs['uncorrelated_gauss_noise'] = True
             nargs['gauss_noise_ampl_x'] = np.sqrt(nargs['gauss_noise_ampl_x']**2+gauss_noise_ampl_x**2)
+            nargs['gauss_noise_ampl_px'] = np.sqrt(nargs['gauss_noise_ampl_px']**2+gauss_noise_ampl_px**2)
             nargs['gauss_noise_ampl_y'] = np.sqrt(nargs['gauss_noise_ampl_y']**2+gauss_noise_ampl_y**2)
-            nargs['gauss_noise_ampl_s'] = np.sqrt(nargs['gauss_noise_ampl_s']**2+gauss_noise_ampl_s**2)
+            nargs['gauss_noise_ampl_py'] = np.sqrt(nargs['gauss_noise_ampl_py']**2+gauss_noise_ampl_py**2)
+            nargs['gauss_noise_ampl_zeta'] = np.sqrt(nargs['gauss_noise_ampl_zeta']**2+gauss_noise_ampl_zeta**2)
+            nargs['gauss_noise_ampl_delta'] = np.sqrt(nargs['gauss_noise_ampl_delta']**2+gauss_noise_ampl_delta**2)
 
         super().__init__(**nargs)
 
