@@ -1,3 +1,8 @@
+// copyright ############################### //
+// This file is part of the Xtrack Package.  //
+// Copyright (c) CERN, 2021.                 //
+// ######################################### //
+
 #ifndef XTRACK_DRIFT_H
 #define XTRACK_DRIFT_H
 
@@ -9,11 +14,11 @@ void Drift_track_local_particle(DriftData el, LocalParticle* part0){
     //start_per_particle_block (part0->part)
 
         double const rpp    = LocalParticle_get_rpp(part);
+        double const rv0v    = 1./LocalParticle_get_rvv(part);
         double const xp     = LocalParticle_get_px(part) * rpp;
         double const yp     = LocalParticle_get_py(part) * rpp;
-        double const dzeta  = LocalParticle_get_rvv(part) -
-                               ( 1. + ( xp*xp + yp*yp ) / 2. );
-    
+        double const dzeta  = 1 - rv0v * ( 1. + ( xp*xp + yp*yp ) / 2. );
+
         LocalParticle_add_to_x(part, xp * length );
         LocalParticle_add_to_y(part, yp * length );
         LocalParticle_add_to_s(part, length);
