@@ -1,3 +1,8 @@
+# copyright ############################### #
+# This file is part of the Xtrack Package.  #
+# Copyright (c) CERN, 2021.                 #
+# ######################################### #
+
 import xobjects as xo
 
 from . import beam_elements as be
@@ -29,7 +34,7 @@ class LineFrozen:
 
         num_elements = len(line.element_names)
 
-        element_data_types = set(ee.XoStruct for ee in line.elements)
+        element_data_types = set(ee._XoStruct for ee in line.elements)
         sorted_element_data_types = sorted(
 			element_data_types, key = lambda cc:cc.__name__)
 
@@ -45,7 +50,7 @@ class LineFrozen:
 
         for ii, (ee, nn) in enumerate(zip(line.elements,
                                       line.element_names)):
-            assert hasattr(ee, 'XoStruct') # is already xobject
+            assert hasattr(ee, '_XoStruct') # is already xobject
             if ee._buffer != line_data._buffer:
                 if ee._xobject._has_refs:
                     raise ValueError(
@@ -54,7 +59,7 @@ class LineFrozen:
                 "Elements containing references should be placed directly in the"
                 " same buffer as the tracker."
                 )
-                ee._move_to(_buffer=line_data._buffer)
+                ee.move(_buffer=line_data._buffer)
 
             line_data[ii] = ee._xobject
 
