@@ -5,15 +5,17 @@
 
 import numpy as np
 import logging
+from functools import partial
 
 from .general import _pkg_root
 from .line_frozen import LineFrozen
 from .base_element import _handle_per_particle_blocks
-from .twiss_from_tracker import (twiss_from_tracker,
+from .twiss import (twiss_from_tracker,
                                  compute_one_turn_matrix_finite_differences,
-                                 find_closed_orbit,
+                                 find_closed_orbit, match_tracker
                                 )
-from .interal_record import (RecordIdentifier, RecordIndex, new_io_buffer,
+from .survey import survey_from_tracker
+from .interal_record import (new_io_buffer,
                              start_internal_logging_for_elements_of_type,
                              stop_internal_logging_for_elements_of_type)
 from .pipeline import PipelineStatus
@@ -367,7 +369,8 @@ class Tracker:
                 "Please rebuild the tracker, for example using `line.build_tracker(...)`.")
 
     def find_closed_orbit(self, particle_co_guess=None, particle_ref=None,
-                          co_search_settings={}, delta_zeta=0):
+                          co_search_settings={}, delta_zeta=0, delta0=None,
+                          continue_on_closed_orbit_error=False):
 
         self._check_invalidated()
 
@@ -384,8 +387,9 @@ class Tracker:
             tracker = self
 
         return find_closed_orbit(tracker, particle_co_guess=particle_co_guess,
-                                 particle_ref=particle_ref,
-                                 co_search_settings=co_search_settings, delta_zeta=delta_zeta)
+                                 particle_ref=particle_ref, delta0=delta0,
+                                 co_search_settings=co_search_settings, delta_zeta=delta_zeta,
+                                 continue_on_closed_orbit_error=continue_on_closed_orbit_error)
 
     def compute_one_turn_matrix_finite_differences(
             self, particle_on_co,
@@ -404,12 +408,16 @@ class Tracker:
         return compute_one_turn_matrix_finite_differences(tracker, particle_on_co,
                                                    steps_r_matrix)
 
-    def twiss(self, particle_ref=None, r_sigma=0.01,
-        nemitt_x=1e-6, nemitt_y=1e-6,
-        n_theta=1000, delta_disp=1e-5, delta_chrom=1e-4,
-        particle_co_guess=None, steps_r_matrix=None,
-        co_search_settings=None, at_elements=None, at_s=None,
+    def twiss(self, particle_ref=None, delta0=None, method='6d',
+        r_sigma=0.01, nemitt_x=1e-6, nemitt_y=1e-6,
+        delta_disp=1e-5, delta_chrom=1e-4,
+        particle_co_guess=None, R_matrix=None, W_matrix=None,
+        steps_r_matrix=None, co_search_settings=None, at_elements=None, at_s=None,
+        values_at_element_exit=False,
+        continue_on_closed_orbit_error=False,
         eneloss_and_damping=False,
+        ele_start=0, ele_stop=None, twiss_init=None,
+        particle_on_co=None,
         matrix_responsiveness_tol=None,
         matrix_stability_tol=None,
         symplectify=False
@@ -417,40 +425,16 @@ class Tracker:
 
         self._check_invalidated()
 
-        if matrix_responsiveness_tol is None:
-            matrix_responsiveness_tol = self.matrix_responsiveness_tol
-        if matrix_stability_tol is None:
-            matrix_stability_tol = self.matrix_stability_tol
+        kwargs = locals().copy()
+        kwargs.pop('self')
 
-        if self.iscollective:
-            logger.warning(
-                'The tracker has collective elements.\n'
-                'In the twiss computation collective elements are'
-                ' replaced by drifts')
-            tracker = self._supertracker
-        else:
-            tracker = self
+        return twiss_from_tracker(self, **kwargs)
 
-        if particle_ref is None:
-            if particle_co_guess is None:
-                particle_ref = self.particle_ref
-
-        if particle_ref is None and particle_co_guess is None:
-            raise ValueError(
-                "Either `particle_ref` or `particle_co_guess` must be provided")
-
-        return twiss_from_tracker(tracker, particle_ref, r_sigma=r_sigma,
-            nemitt_x=nemitt_x, nemitt_y=nemitt_y,
-            n_theta=n_theta, delta_disp=delta_disp, delta_chrom=delta_chrom,
-            particle_co_guess=particle_co_guess,
-            steps_r_matrix=steps_r_matrix,
-            co_search_settings=co_search_settings,
-            at_elements=at_elements, at_s=at_s,
-            eneloss_and_damping=eneloss_and_damping,
-            matrix_responsiveness_tol=matrix_responsiveness_tol,
-            matrix_stability_tol=matrix_stability_tol,
-            symplectify=symplectify)
+    def survey(self,X0=0,Y0=0,Z0=0,theta0=0,phi0=0,psi0=0):
+        return survey_from_tracker(self,X0=X0,Y0=Y0,Z0=Z0,theta0=theta0,phi0=phi0,psi0=psi0)
 
+    def match(self, vary, targets, **kwargs):
+        return match_tracker(self, vary, targets, **kwargs)
 
     def filter_elements(self, mask=None, exclude_types_starting_with=None):
 
@@ -560,6 +544,10 @@ class Tracker:
         else:
             self._enable_pipeline_hold = value
 
+    @property
+    def _context(self):
+        return self._buffer.context
+
     def configure_radiation(self, mode=None):
 
         self._check_invalidated()
@@ -673,6 +661,10 @@ class Tracker:
                     }
                     increment_at_element(&lpart);
                 } // for elements
+                if (flag_monitor==2){
+                    // End of turn (element-by-element mode)
+                    ParticlesMonitor_track_local_particle(tbt_monitor, &lpart);
+                }
                 if (flag_end_turn_actions>0){
                     if (isactive){
                         increment_at_turn(&lpart, flag_reset_s_at_end_turn);
@@ -717,24 +709,19 @@ class Tracker:
         # Random number generator init kernel
         kernels.update(self.particles_class._kernels)
 
-        self.particles_class._XoStruct._extra_c_sources.append(self.local_particle_src)
-
-        try:
-            # Compile!
-            context.add_kernels(
-                [source_track],
-                kernels,
-                extra_headers=headers,
-                extra_classes=self.element_classes,
-                apply_to_source=[_handle_per_particle_blocks],
-                save_source_as=save_source_as,
-                specialize=True,
-                compile=compile
-            )
-            self.particles_class._XoStruct._extra_c_sources.pop()
-        except Exception as e:
-            self.particles_class._XoStruct._extra_c_sources.pop()
-            raise e
+        # Compile!
+        context.add_kernels(
+            [source_track],
+            kernels,
+            extra_headers=headers,
+            extra_classes=self.element_classes,
+            apply_to_source=[
+                partial(_handle_per_particle_blocks,
+                        local_particle_src=self.local_particle_src)],
+            save_source_as=save_source_as,
+            specialize=True,
+            compile=compile
+        )
 
         self.track_kernel = context.kernels.track_line
 
@@ -1247,7 +1234,7 @@ class Tracker:
         elif turn_by_turn_monitor == 'ONE_TURN_EBE':
             (_, monitor, buffer_monitor, offset_monitor
                 ) = self._get_monitor(particles, turn_by_turn_monitor=True,
-                                      num_turns=len(self.line.elements))
+                                      num_turns=len(self.line.elements)+1)
             monitor.ebe_mode = 1
             flag_monitor = 2
         elif isinstance(turn_by_turn_monitor, self.particles_monitor_class):
@@ -1265,8 +1252,6 @@ class Tracker:
         return start_internal_logging_for_elements_of_type(self,
                                                     element_type, capacity)
 
-        self._check_invalidated()
-
     def stop_internal_logging_for_elements_of_type(self, element_type):
         self._check_invalidated()
         stop_internal_logging_for_elements_of_type(self, element_type)
